# w.c.s. - web application for online forms
# Copyright (C) 2005-2024  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import subprocess

from quixote import get_publisher, get_response

from wcs.qommon import _, template
from wcs.qommon.afterjobs import AfterJob

RETURNCODE_MAP = {
    0: _('No virus found'),
    1: _('Virus(es) found'),
    2: _('An error occurred'),
}


class PickableClamD:
    @property
    def clamd(self):
        if getattr(self, '_clamd', None) is None:
            self._clamd = {}
        return self._clamd

    def init_clamd(self):
        if get_publisher().has_site_option('enable-clamd') and 'waiting_scan' not in self.clamd:
            self.clamd['waiting_scan'] = True

    def is_waiting_for_scan(self):
        if not get_publisher().has_site_option('enable-clamd'):
            return False
        return self.clamd.get('waiting_scan', False)

    def has_malware(self):
        if not get_publisher().has_site_option('enable-clamd'):
            return False
        if 'waiting_scan' not in self.clamd:
            # files uploaded before clamd was enabled
            return False
        if self.clamd['waiting_scan']:
            # file not scanned yet
            return False
        if self.clamd['returncode'] == 0:
            return False
        # scan dit not run properly or detected a virus
        return True

    def allow_download(self):
        return not (self.is_waiting_for_scan() or self.has_malware())


class ClamDScanJob(AfterJob):
    def __init__(self, formdata, **kwargs):
        super().__init__(
            formdef_class=formdata.formdef.__class__, formdef_id=formdata.formdef.id, formdata_id=formdata.id
        )

    def execute(self):
        formdef = self.kwargs['formdef_class'].get(self.kwargs['formdef_id'])
        formdata = formdef.data_class().get(self.kwargs['formdata_id'])
        store = False
        for file_data in formdata.get_all_file_data(with_history=False):
            if file_data.clamd.get('waiting_scan', False):
                path = (
                    file_data.get_fs_filename()
                    if hasattr(file_data, 'get_fs_filename')
                    else file_data.get_file_path()
                )
                clamd = subprocess.run(['clamdscan', '--fdpass', path], check=False)
                file_data.clamd['returncode'] = clamd.returncode
                file_data.clamd['waiting_scan'] = False
                store = True
        if store:
            formdata._store_all_evolution = True
            formdata.store()


def add_clamd_scan_job(formdata):
    if get_publisher().has_site_option('enable-clamd'):
        job = get_response().add_after_job(ClamDScanJob(formdata=formdata))
        job.store()


def malware_response(file_data):
    return template.QommonTemplateResponse(
        templates=['wcs/malware.html'],
        context={
            'is_waiting_for_scan': file_data.is_waiting_for_scan(),
            'has_malware': file_data.has_malware(),
        },
        is_django_native=True,
    )
